#function to remove ThorApps Event receivers
function Remove-ThorAppsEventReceiversFromSiteCollections($targetSites)
{
	foreach($site in $targetSites)
	{
		$webs = Get-SPWeb -Site $site -Limit ALL
		foreach($web in $webs)
		{
			$web = Get-SPWeb $web.Url
			$listIndex = 0
			$lists = $web.Lists
			$listCount = $lists.Count
			Write-Host "Removing ThorApps Event Receivers on lists in web: " $web.Url -f yellow
			while($listIndex -lt $listCount)
			{
				$list = $lists[$listIndex]
				$eventReceiverColl = $list.EventReceivers			
				$eventReceiverCount = $list.EventReceivers.Count
				$thorAppsReceiverColl = @()
				for($intIndex = $eventReceiverCount -1; $intIndex -gt -1; $intIndex-)
				{
					$objReceiver = $eventReceiverColl[$intIndex]
					if($objReceiver.Class -like "Thor.SPF.ReflexReporting*")
					{
						$thorAppsReceiverColl += $objReceiver
					}
				}
				Delete-ThorAppsEventReceivers -thorAppsEventReceiverColl $thorAppsReceiverColl -ErrorAction Stop
				$listIndex = $listIndex + 1
			}

			Write-Host "Removing ThorApps Event Receivers from web: " $web.Url -f yellow
			$eventReceiverColl = $web.EventReceivers			
			$eventReceiverCount = $web.EventReceivers.Count
			$thorAppsReceiverColl = @()
			for($intIndex = $eventReceiverCount -1; $intIndex -gt -1; $intIndex-)
			{
				$objReceiver = $eventReceiverColl[$intIndex]
				if($objReceiver.Class -like "Thor.SPF.ReflexReporting*")
				{
					$thorAppsReceiverColl += $objReceiver
				}
			}
			Delete-ThorAppsEventReceivers -thorAppsEventReceiverColl $thorAppsReceiverColl -ErrorAction Stop
		}

		Write-Host "Removing ThorApps Event Receivers from site: " $site -f yellow
		$spSite = Get-SPSite $site
		$eventReceiverColl = $spSite.EventReceivers			
		$eventReceiverCount = $spSite.EventReceivers.Count
		$thorAppsReceiverColl = @()
		for($intIndex = $eventReceiverCount -1; $intIndex -gt -1; $intIndex-)
		{
			$objReceiver = $eventReceiverColl[$intIndex]
			if($objReceiver.Class -like "Thor.SPF.ReflexReporting*")
			{
				$thorAppsReceiverColl += $objReceiver
			}
		}
		Delete-ThorAppsEventReceivers -thorAppsEventReceiverColl $thorAppsReceiverColl -ErrorAction Stop
	}
} 

#function Delete ThorApps Event Receivers
function Delete-ThorAppsEventReceivers
{
	param
	(
		$thorAppsEventReceiverColl
	)
    $intIndex = 0
    $count = $thorAppsEventReceiverColl.Count
    while($intIndex -lt $count)
    {
        $thorAppsEventReceiverColl[$intIndex].Delete()
        Write-Host "Deleted Event Receiver: " $thorAppsEventReceiverColl[$intIndex].Class -f green
        $intIndex = $intIndex + 1
    }
}

#function Disable ThorApps Site and Site Collection features
function Deactivate-ThorAppsFeatures($targetSites)
{
    foreach($site in $targetSites)
    {
        Write-Host "About to Deactivate Features on site: " $site -f cyan	
        $siteFeatures = Get-SPFeature -Site $site -Limit ALL | Where-Object {$_.DisplayName -like "Thor*"} | Where-Object {$_.Scope -eq "SITE"}
        if ($siteFeatures -ne $null)
        {
        	foreach ($siteFeature in $siteFeatures)
	        {
            		Write-Host "Disabling Site Collection feature " $siteFeature.DisplayName ", at scope: " $siteFeature.Scope ", on site: " $site -f yellow
            		Disable-SPFeature -Url $site.Url -Identity $siteFeature.Id -Confirm:$false
            }
        }
        $webs = Get-SPWeb -Site $site -Limit ALL
        foreach($web in $webs)
        {
		    $webFeatures = Get-SPFeature -Web $web.Url -Limit ALL | Where-Object {$_.DisplayName -like "Thor*"} | Where-Object {$_.Scope -eq "WEB"}
		    if ($webFeatures -ne $null)
		    {
			    foreach ($webFeature in $webFeatures)
			    {
		                Write-Host "Disabling Site feature: " $webFeature.DisplayName ", at scope: " $webFeature.Scope ", on site: " $site -f yellow
        		        Disable-SPFeature -Url $web.Url -Identity $webFeature.Id -Confirm:$false
			    }
		    }
        }
    }
}

function DoCleanUp
{
    Write-Host "Starting Clean up"
    $webAppColl = Get-SPWebApplication #-IncludeCentralAdministration
    Write-Host $("Found " + $webAppColl.Count.ToString() + " web applications")

    Foreach ($webApp in $webAppColl)
    {
        Write-Host $("Found " + $webApp.Sites.Count.ToString() + " site collections in web app")
        Remove-ThorAppsEventReceiversFromSiteCollections -targetSites $webApp.Sites
        Deactivate-ThorAppsFeatures -targetSites $webapp.Sites
    }
    Write-Host "Clean up finished"
}

DoCleanUp